<?php
/* $Id: class_graph.inc,v 1.11 2003/08/02 12:37:04 dijkstra Exp $ */

/*
 * Copyright (c) 2003 Willem Dijkstra
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Graph class
 * 
 * Generates rrdtool recipes by rewriting a template using a joined set of all
 * variables (layout vars, graph vars, session vars - in order of precendence).
 */
require_once('class_session.inc');
require_once('class_cache.inc');
require_once('setup.inc');
require_once('class_vars.inc');
require_once('graph_defaults.inc');

class Graph {
    var $template;
    var $vars;
    var $graph_vars;
    var $group_vars;
    var $definition;
    var $url;

    function Graph(&$group_vars) {
	$this->group_vars = $group_vars;
    }

    function set_graph_vars(&$graph_vars) {
	$this->graph_vars = $graph_vars;
    }

    function _makevars() {
	global $session;

	$this->definition = '';
	unset($this->url);

	/* create local state by combining group and local graph args */
	$this->vars = new Vars();
	$this->vars->addvars($session->getvars());
	$this->vars->addvars($this->graph_vars);
	$this->vars->addvars($this->group_vars);

	if ($this->vars->defp('rrdfile')) { 
	    $f = $this->vars->get('rrdfile');
	    if (is_file($f) && 
		preg_match("/^(.*\/)?(cpu|if|io|mem|pf|proc|debug|mbuf)(_([a-z]+))?([0-9]?).rrd$/", 
			   $f, $match)) {
		$rrdtype = $match[2];
		$rrdwhat = $match[4];
		$rrdwhich = $match[5];

		if (!$this->vars->defp('rrdtype')) {
		    $this->vars->set('rrdtype', $rrdtype);
		}

		if (!$this->vars->defp('rrdwhat')) {
		    $this->vars->set('rrdwhat', $rrdwhat);
		}

		if (!$this->vars->defp('rrdwhich')) {
		    $this->vars->set('rrdwhich', $rrdwhich);
		}

		if (!$this->vars->defp('title')) {
		    $this->vars->set('title', $rrdtype.'('.$rrdwhat.$rrdwhich.') of '.$this->vars->get('name'));
		}
	    }
	}

	if (!$this->vars->defp('template')) {
	    if (!$this->vars->defp('rrdtype')) {
		$this->_display();
		runtime_error('graph: trying to load default template, but rrdtype is not set');
	    }
	    $template = get_default_template($this->vars->get('rrdtype'));
	    $this->template = preg_split("/\n/", $template);
	} else {
	    $this->template = preg_split("/\n/", $this->vars->get('template'));
	}
    }
    
    function parse(&$lexer) {
	$this->graph_vars = new Vars();
	$this->graph_vars->parse($lexer);
    }

    function render() {
	$this->_save();
			print '
<div class="graph">
<img src="graph.php?'. $this->_url() . '">
</div>
';
    }

    function _url() {
	if (!isset($this->url)) {
	    runtime_error("graph: internal error : need to save the graph before an url can be requested");
	    return '';
	} else {
	    return $this->url;
	}
    }

    
    function _save() {
	global $cache;
	global $symon;

	$this->_makevars();

	if (isset($symon['graph_debug'])) { 
	    $this->_display();
	    print "<pre>";
	    if (isset($this->template)) {
		print "\xa template = ";
		foreach ($this->template as $line) {
		    print "\xa    '$line'";
		}
	    }
	}

	$this->_preprocess();

	if (isset($symon['graph_debug'])) { 
	    print "\xa preprocessed template = ";
	    foreach ($this->definition as $line) {
		print "\xa    '$line'";
	    }
	}

	$this->_constrain();

	if (isset($symon['graph_debug'])) {
	    print "\xa constrained template = ";
	    foreach ($this->definition as $line) {
		print "\xa    '$line'";
	    }
	}
	    
	$seed = $this->vars->tostring();
	$graph_command = implode("\n", $this->definition); 
	$this->url = $cache->insert($graph_command, '.txt', $seed);
	
	if (isset($symon['graph_debug'])) {
	    print "\xa url = ". $this->url. "</pre>";
	}
    }

    /* fill template with variables */
    function _preprocess() {
	$definition = '';
	if (!isset($this->template) || !is_array($this->template)) {
	    return;
	}
	reset($this->template);

	foreach ($this->template as $t) {
	    if (preg_match("/%([^%0-9][^%]*)%/", $t)) {
		$items = preg_split("/%([^%0-9][^%]*)%/", $t, -1, PREG_SPLIT_DELIM_CAPTURE);
		$definition .= array_shift($items);
		while ($key = array_shift($items)) {
		    if ($this->vars->defp($key)) {
			$definition .= $this->vars->get($key);
		    } else {
			$definition .= '%'.$key.'%';
		    }
		    $definition .= array_shift($items);
		}
	    } else {
		$definition .= $t;
	    }

	    $definition .= "\n";
	}

	$this->definition = preg_split("/\n/", $definition);
    }
    /* constrain arguments */
    function _constrain() {
	global $symon;

	$definition = '';
	if (!is_array($this->definition)) {
	    return 1;
	}

	reset($this->definition);
	foreach ($this->definition as $t) {
	    if (preg_match("/^([^-]*)(-[^ ]+) ([^ ]+)(.*)$/", $t)) {
		while (preg_match("/^([^-]*)(-[^ ]+) ([^ ]+)(.*)$/", $t, $match)) {
		    $definition .= $match[1];
		    foreach ($symon['constraints'] as $k => $v) {
			if ($k == $match[2]) {
			    if (is_array($symon['constraints'][$k])
				&& isset($symon['constraints'][$k]["max"])
				&& ($match[3] > $symon['constraints'][$k]["max"])) {
				$match[3] = $symon['constraints'][$k]["max"];
			    }
			    if (is_array($symon['constraints'][$k])
				&& isset($symon['constraints'][$k]["min"])
				&& ($match[3] < $symon['constraints'][$k]["min"])) {
				$match[3] = $symon['constraints'][$k]["min"];
			    }
			}
		    }
		    $definition .= $match[2].' '.$match[3];
		    $t = $match[4];
		}
	    }
	    $definition .= $t."\n";
	}
	$this->definition = preg_split("/\n/", $definition);
    }
    
    function _display() {
	print "<pre>\xa graph " . $this->graph_vars->tostring() . ";</pre>";
    }
}
    
?>